/*
 * Java Othello Openings Applet
 * Copyright (C) 2000 Bill Lorton
 * email: hogarth1@yahoo.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.awt.*;

/**
 *
 * Board class for the Java Othello Openings Applet.
 *
 * Handles drawing the board and making individual moves on the Othello board.
 *
 * @version 1.00
 * @author Bill Lorton
 */

class Board extends Panel {
  private Othello _app;

  private String _col_strs[] = {"A","B","C","D","E","F","G","H"};

  private Image []_disc_imgs;

  private Font _col_font,_row_font;

  static private final int SQUARE_SIZE  = 40;
  static private final int BORDER_WIDTH = 25;
  static private final int EMPTY = 0;
  static private final int WHITE = 1;
  static private final int BLACK = 2;

  /**
   *
   * A 2x2 array of disc types (EMPTY,WHITE,BLACK).
   *
   */
  protected int [][]_board;


  /**
   *
   * Used to determine where to draw the last move indicator.
   *
   */
  protected int _last_r,_last_c;

  /**
   *
   * Returns the position on the board in pixels for the given row or column.
   *
   * @param rc an x or y index for the board. valid values are 0-7
   * @return the x or y position in pixels for the row or column
   */
  protected int _squarePos(int rc)
  {
    return rc*SQUARE_SIZE+BORDER_WIDTH;
  }

  /**
   *
   * Flips the discs intersecting the placed disc in one direction. For example
   * with a row_inc of -1 and a col_inc of -1, discs are flipped diagonally 
   * from the placed disc towards the upper left.  The placed disc is never
   * modified.
   *
   * @param r the row of the disc that was placed
   * @param c the column of the disc that was placed
   * @param row_inc the row increment value
   * @param col_inc the column increment value
   * @return number of discs that were flipped
   *
   */
  protected int _flip(int r,int c,int row_inc,int col_inc)
  {
    int n_flipped   = 0;
    int placed_disc = _board[r][c];

    int rr = r + row_inc; // start at one unit away from r,c
    int cc = c + col_inc;

    // search in the given direction for a disc of the same color (w or b)
    // as the placed disc.  that will give us the range of discs to flip

    for (;;) {
      if (rr<0 || rr>7 || cc<0 || cc>7) { // stop if invalid position
        rr = -1;
        break;
      }

      int disc_type = _board[rr][cc];  // get disc type

      if (disc_type==EMPTY) { // stop if you get to an empty position
        rr = -1;
        break;
      }

      if (disc_type==placed_disc)  // found a disc of the same type (w or b)
        break;

      rr += row_inc;
      cc += col_inc;
    }

    if (rr!= -1) {    // if a range of discs to flip was found.
      rr -= row_inc;    // increment backwards from the found disc
      cc -= col_inc;    // back to the placed disc.

      for (;;) {
        if (rr==r && cc==c)  // we are on the placed disc, stop.
	  break;

        _board[rr][cc] = placed_disc;  // replace the current disc with
        ++n_flipped;                   // one of the correct color.
        rr -= row_inc;
        cc -= col_inc;
      }
    }

    return n_flipped;
  }


  /**
   *
   * Draws the board.
   *
   * @param g the graphics context
   *
   */
  protected void _paint(Graphics g)
  {
    // draw discs

    for (int r=0;r<8;++r) {
      for (int c=0;c<8;++c) {
        int cx = _squarePos(c);
        int ry = _squarePos(r);

        g.drawImage(_disc_imgs[_board[r][c]],cx,ry,this);

        if (r==_last_r && c==_last_c) { // draw last move indicator
          int x = cx+SQUARE_SIZE-9;
          int y = ry+SQUARE_SIZE-9;

          g.setColor(Color.yellow);
          g.fillOval(x,y,4,4);
          g.setColor(Color.red);
          g.drawOval(x,y,4,4);
        }
      }
    }

    // draw border, and the row and column labels

    g.setColor(Color.black);

    FontMetrics metrics = Toolkit.getDefaultToolkit().getFontMetrics(_row_font);
    int         height  = metrics.getAscent();

    int half_square = SQUARE_SIZE/2;
    int half_height = height/2;
    int half_border = BORDER_WIDTH/2;

    g.setFont(_col_font);

    for (int i=0;i<8;++i) {
      int x = 
        BORDER_WIDTH+i*SQUARE_SIZE+half_square -
          metrics.stringWidth(_col_strs[i])/2;

      g.drawString(_col_strs[i],x,half_border+half_height);
    }

    g.setFont(_row_font);

    for (int i=0;i<8;++i) {
      String s = Integer.toString(i+1);
      int    y = BORDER_WIDTH+i*SQUARE_SIZE+half_square+height/2;

      g.drawString(s,half_border-metrics.stringWidth(s)/2,y);
    }
  }


  /**
   *
   * Board class constructor.
   *
   * @param app the othello applet class which is used to load the board's
   * images
   *
   */
  public Board(Othello app)
  {
    _app = app;
  }


  /**
   *
   * Calculates the size of the board.
   *
   * @return the minimum size of the board
   *
   */
  public Dimension getMinimumSize()
  {
    int size = SQUARE_SIZE*8+BORDER_WIDTH;
    return new Dimension(size,size);
  }


  /**
   *
   * Creates the needed fonts, loads the disc images and initializes the
   * data structures.  The Othello class is used to load the images.
   *
   */
  public void init()
  {
    _col_font = new Font("TimesRoman",Font.BOLD,14);
    _row_font = new Font("TimesRoman",Font.BOLD,16);
    _disc_imgs = new Image[3];

    _disc_imgs[EMPTY] = _app.getImage(_app.getCodeBase(),"ndisc.jpg");
      // no disc (an empty square)
    _disc_imgs[WHITE] = _app.getImage(_app.getCodeBase(),"wdisc.jpg");
      // white disc
    _disc_imgs[BLACK] = _app.getImage(_app.getCodeBase(),"bdisc.jpg");
      // black disc

    _board = new int[8][8];
    clear();
    setLayout(null);
    setSize(345,345);
  }


  /**
   *
   * Draws the board.
   *
   * @param g the graphics context
   */
  public void paint(Graphics g)
  {
    _paint(g);
  }

  /**
   *
   * Draws the board.
   *
   * @param g the graphics context
   *
   */
  public void update(Graphics g)
  {
    _paint(g);
  }

  /**
   *
   * Sets the data structures back to their initial state.
   *
   */
  public void clear()
  {
    for (int r=0;r<8;++r) {
      for (int c=0;c<8;++c) {
        _board[r][c] = EMPTY;
      }
    }

    // place the initial discs

    _board[3][3] = WHITE;
    _board[3][4] = BLACK;
    _board[4][3] = BLACK;
    _board[4][4] = WHITE;
    _last_r = _last_c = -1;
  }


  /**
   *
   * Makes one move.  The position specified for the move cannot already
   * have a disc.  The disc placement must flip at least one existing disc.
   *
   * @param r the row of the move
   * @param c the column of the move
   * @param white_move true if a white disc is being placed, false if a 
   * black disc is being placed
   *
   */
  public void placeDisc(int r,int c,boolean white_move)
  {
    if (_board[r][c]!=EMPTY) {
      System.out.println("Invalid move, "+r+","+c+" is already occupied.");
      return;
    }

    // place the disc

    _board[r][c] = white_move ? WHITE : BLACK;

    // perform the flips

    int n_flipped = 0;

    n_flipped += _flip(r,c,0,-1);  // left
    n_flipped += _flip(r,c,0,1);   // right
    n_flipped += _flip(r,c,-1,0);  // up
    n_flipped += _flip(r,c,1,0);   // down
    n_flipped += _flip(r,c,-1,-1); // up-left
    n_flipped += _flip(r,c,-1,1);  // up-right
    n_flipped += _flip(r,c,1,-1);  // down-left
    n_flipped += _flip(r,c,1,1);   // down-right


    if (n_flipped==0) {
      System.out.println(
                   "Invalid move, placing a disc at "+r+","+c+" flips no discs."
                 );
    }

    _last_r = r;
    _last_c = c;
  }
}

